/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.sun.idm.rpc.spml2.extended;

import com.sun.idm.rpc.spml2.BasicSpml2Executor;
import com.waveset.session.Session;
import org.openspml.v2.msg.spml.Request;
import org.openspml.v2.msg.spml.Response;
import org.openspml.v2.msg.spml.StatusCode;
import org.openspml.v2.msg.spml.LaunchProcessRequest;
import org.openspml.v2.msg.spml.LaunchProcessResponse;
import org.openspml.v2.util.Spml2Exception;
import com.sun.idm.logging.trace.Trace;
import com.sun.idm.logging.trace.TraceManager;
import com.waveset.object.ResultItem;
import com.waveset.object.WavesetResult;
import com.waveset.session.SessionUtil;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.openspml.v2.msg.OpenContentAttr;
import org.openspml.v2.msg.OpenContentElement;
import org.openspml.v2.msg.spml.ErrorCode;
import org.openspml.v2.msg.spml.Extensible;
import org.openspml.v2.profiles.dsml.DSMLAttr;
import org.openspml.v2.profiles.dsml.DSMLValue;

/**
 *
 * @author ricky
 */
public class LaunchRequestExecutor extends BasicSpml2Executor {

    private static final Trace TRACE = TraceManager.getTrace(
            LaunchRequestExecutor.class.getName());

    @Override
    protected void validateRequest(Request request, Response response)
            throws Spml2Exception {
        if (!(request instanceof LaunchProcessRequest)) {
            failRequest(response, ErrorCode.MALFORMED_REQUEST, "Expected a LaunchProcessRequest.");
        }
        LaunchProcessRequest req = (LaunchProcessRequest) request;
        if (req.getPsoID() == null || req.getPsoID().getID() == null) {
            failRequest(response, ErrorCode.INVALID_IDENTIFIER, "Expected a Workflow in PSO.");
        }
    }

    @Override
    protected Response createResponse(Request request) throws Spml2Exception {
        return new LaunchProcessResponse(null, StatusCode.SUCCESS,
                request.getRequestID(), null, null);
    }

    @Override
    protected Response executeRequest(Request request, Response response,
            Session ses) throws Spml2Exception {
        String METHOD = "executeRequest";
        TRACE.entry1(METHOD);
        // validate the request
        validateRequest(request, response);
        // initialize req and res
        LaunchProcessRequest req = (LaunchProcessRequest) request;
        LaunchProcessResponse res = (LaunchProcessResponse) response;
        // get the name of the workflow
        String workflow = req.getPsoID().getID();
        // construct a map with the variables
        Map<String, Object> inputs = new HashMap<String, Object>();
        OpenContentElement[] elements = null;
        OpenContentAttr[] attrs = null;
        if (req.getData() != null) {
            elements = req.getData().getOpenContentElements();
            attrs = req.getData().getOpenContentAttrs();
        }
        // first add attrs in a usual way
        if (attrs != null) {
            for (int i = 0; i < attrs.length; i++) {
                inputs.put(attrs[i].getName(), attrs[i].getValue());
            }
        }
        // in case DSMLAttr exists override values (multivalue as List)
        if (elements != null) {
            for (int i = 0; i < elements.length; i++) {
                if (elements[i] instanceof DSMLAttr) {
                    // DSMLAttr trated special for multivalues
                    DSMLAttr attr = (DSMLAttr) elements[i];
                    if (attr.getValues().length == 1) {
                        inputs.put(attr.getName(), attr.getValues()[0].getValue());
                    } else if (attr.getValues().length > 1) {
                        List<String> values = new ArrayList<String>(attr.getValues().length);
                        for (DSMLValue value: attr.getValues()) {
                            values.add(value.getValue());
                        }
                        inputs.put(attr.getName(), values);
                    }
                }
            }
        }
        TRACE.info4(METHOD, "Workflow to execute: " + workflow);
        TRACE.info4(METHOD, "inputs: " + inputs);
        // launch a workflow with this name and inputs
        WavesetResult result = null;
        try {
            result = SessionUtil.launchProcess(ses, workflow,
                    "SPML Launch Process: " + workflow, inputs);
        } catch (Exception e) {
            TRACE.caught1(METHOD, e);
            res.setStatus(StatusCode.FAILURE);
            res.addErrorMessage("Exception invoking Workflow ("
                    + e.getClass().toString() + "): " + e.getMessage());
            res.setError(ErrorCode.CUSTOM_ERROR);
            return res;
        }
        // assign the data response
        // set the status
        res.setStatus(result.hasError() ? StatusCode.FAILURE : StatusCode.SUCCESS);
        // set the error cause
        if (result.hasError()) {
            res.setError(ErrorCode.CUSTOM_ERROR);
        }
        // assign all the errors
        List<String> errors = result.getErrors();
        if (errors != null) {
            for (String error : errors) {
                res.addErrorMessage(error);
            }
        }
        // assign all the waveset action results data
        List<ResultItem> items = result.getResults();
        if (items != null) {
            Extensible data = new Extensible();
            for (ResultItem item : items) {
                if (ResultItem.DATA.equals(item.getType())) {
                    if (item.getValue() instanceof List) {
                        List<Object> list = (List) item.getValue();
                        DSMLAttr attr = new DSMLAttr(item.getName(), "");
                        attr.clearValues();
                        if (list != null) {
                            for (Object value : list) {
                                if (value != null) {
                                    attr.addValue(new DSMLValue(value.toString()));
                                }
                            }
                        }
                        TRACE.info4(METHOD, "Adding data -> " + item.getName() + ":" + item.getValue().toString());
                        if (attr.getValues().length > 0) {
                            data.addOpenContentElement(attr);
                        }
                    } else if (item.getValue() != null) {
                        TRACE.info4(METHOD, "Adding data -> " + item.getName() + ":" + item.getValue().toString());
                        data.addOpenContentElement(
                                new DSMLAttr(item.getName(), item.getValue().toString()));
                    }
                }
            }
            res.setData(data);
        }
        return res;
    }
}
